
From: Nick Piggin <nickpiggin@yahoo.com.au>

Jack Steiner reported this to have fixed his problem (bad colouring):
"The patches fix both problems that I found - bad
 coloring & excessive pages in pagesets."

In most workloads this is not likely to be such a pronounced problem,
however it should help corner cases.  And avoiding powers of 2 in these
types of memory operations is always a good idea.

Signed-off-by: Nick Piggin <nickpiggin@yahoo.com.au>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 mm/page_alloc.c |   12 ++++++++++++
 1 files changed, 12 insertions(+)

diff -puN mm/page_alloc.c~mm-pcp-use-non-powers-of-2-for-batch-size mm/page_alloc.c
--- 25/mm/page_alloc.c~mm-pcp-use-non-powers-of-2-for-batch-size	2005-04-26 05:06:18.546420992 -0700
+++ 25-akpm/mm/page_alloc.c	2005-04-26 05:06:18.554419776 -0700
@@ -1671,6 +1671,18 @@ static void __init free_area_init_core(s
 		if (batch < 1)
 			batch = 1;
 
+		/*
+		 * Clamp the batch to a 2^n - 1 value. Having a power
+		 * of 2 value was found to be more likely to have
+		 * suboptimal cache aliasing properties in some cases.
+		 *
+		 * For example if 2 tasks are alternately allocating
+		 * batches of pages, one task can end up with a lot
+		 * of pages of one half of the possible page colors
+		 * and the other with pages of the other colors.
+		 */
+		batch = (1 << fls(batch + batch/2)) - 1;
+
 		for (cpu = 0; cpu < NR_CPUS; cpu++) {
 			struct per_cpu_pages *pcp;
 
_
